      +=================-=========-=========-=========*=========-=========-=========-=================+
      ||                                  HIGH PRECISION COMPUTATION                                 ||
      ||                                     (the Decimal type)                                      ||
      +=================-=========-=========-=========*=========-=========-=========-=================+


   Python 3.x has unlimited integer precision (yea!)--e.g.,

        print(123456**9)
        output: 6662079557313332402329122047690156142804074496

   But, # precision is limited with floats:

        print(123.456**9)
        output: 6.66207955731e+18 <--boo!

        from math import factorial
        print (factorial(55))
                          # output: 12696403353658275925965100847566516959580321051449436762275840000000000000
        print (factorial(55)/2)
                          #output: 6.348201676829138e+72 <--boo!
   However, by importing the decimal module, we can do better.

#---Method 1. Normal
    from math import sqrt
    print("Ans =", sqrt(2))
    # output: Ans = 1.4142135623730951 # 16 correct digits. The last digit is incorrect.

#---Method 2. Misleading.
    print("Ans = %1.63f"%sqrt(2))      # 16 correct digits. The last digit is incorrect. Many bogus digits.
    # output: Ans = 1.414213562373095145474621858738828450441360473632812500000000000

#---Method 3. Super accurate.
    from decimal import Decimal, getcontext
    getcontext().prec = 63 # The Decimal default precision is 28 sig figs.
    print('Ans =', Decimal(2).sqrt()) # 62 correct digits with the 63rd digit rounded.
    # output:   Ans = 1.414 2135 6237 3095 0488 0168 8724 2096 9807 8569 6718 7537 6948 0731 7667 974
    # Internet:       1.414 2135 6237 3095 0488 0168 8724 2096 9807 8569 6718 7537 6948 0731 7667 9737

#--Examples.

    from math    import ceil, floor
    from decimal import Decimal, getcontext

    print('Ans  1.', Decimal(1)/Decimal(7) )
    # output: Ans  1. 0.1428571428571428571428571429 # The Decimal default precision is 28 sig figs.

    getcontext().prec = 63 # We change the default to 63.

    print('Ans  2.', Decimal(1)/Decimal(7)) # Correct!
    # output: Ans  2. 0.142857142857142857142857142857142857142857142857142857142857143

    print('Ans  3.', Decimal(1/7)) # Bad idea!
    # output: Ans  3. 0.142857142857142849212692681248881854116916656494140625 # WRONG answer.

    print('Ans  4.', Decimal.from_float(1/7)) # Bad idea!
    # output: Ans  4. 0.142857142857142849212692681248881854116916656494140625 # WRONG answer.

    print('Ans  5.', Decimal(7).sqrt())
    # output: Ans  2. 2.64575131106459059050161575363926042571025918308245018036833446

    print('Ans  6.', Decimal(7).exp()) # inverse of ln()
    # output: Ans  3. 1096.63315842845859926372023828812143244221913483361314378273924

    print('Ans  7.', Decimal(7).ln())  # inverse of exp()
    # output: Ans  4. 1.94591014905531330510535274344317972963708472958186118845939015


    print('Ans  8.', Decimal(7).log10())
    # output: Ans  5. 0.845098040014256830712216258592636193483572396323965406503634954

    print('Ans  9.', ceil (Decimal(7).log10()))
    # output: Ans  6. 1

    print('Ans 10.', floor(Decimal(7).log10()))
    # output: Ans  7. 0

    print('Ans 11.', round(Decimal(7).log10()))
    # output: Ans  8. 1

##  print('Ans 12.', Decimal(7).sin())  # ERROR Yikes, trig is NOT supported.

    print('Ans 13.', Decimal("-.0123")) # Quotes are required for float numbers.
    # output: Ans 12. -0.0123

    x = 7 # integers are Decimal type (unlike floats)
    print('Ans 14.', Decimal(x))
    # output: Ans 14. 7

##    print('Ans 15.', Decimal(1/7) ) # ERROR
#-------------------------------------------------------------------------------------------------------------

   We can look up the formulas for trig functions on the Internet and make them work in Decimal mode. Note the
capital 'S' in sin and capital 'C' in cos. We don't want to overwrite the built-in functions.


def Cos(x): # Note the capital "C".
# Definition: cos(x) = 1 - x^2/2! + x^4/4! - x^6/6! + ...
    from decimal import Decimal, getcontext
    getcontext().prec += 2 # clever
    sum = Decimal("1")
    i, lasts, s, fact, num, sign = 0, 0, 1, 1, 1, 1
    while s != lasts:
        lasts = s
        i += 2
        fact *= i * (i-1)
        num *= x * x
        sign *= -1
        s += num / fact * sign
    getcontext().prec -= 2 # clever
    return +s
#-------------------------------------------------------------------------------------------------------------

def Sin(x):
# Definition: sin(x) = x - x^3/3! + x^5/5! - x^7/7! +  x^9/9! - x^11/11! +...
    from decimal import Decimal, getcontext
    getcontext().prec += 2
    i, lasts, s, fact, num, sign = 1, 0, x, 1, x, 1
    while s != lasts:
        lasts = s
        i += 2
        fact *= i * (i-1)
        num *= x * x
        sign *= -1
        s += num / fact * sign
    getcontext().prec -= 2
    return +s
