"""   +=================-=========-=========-=========*=========-=========-=========-=================+
      ||                                      FORMATTING OUTPUT                                      ||
      +=================-=========-=========-=========*=========-=========-=========-=================+


   Formatting ints, floats, and strings is a little tricky. The symbol % is officially called the "string
formatting operator" in Python. Expressions like %s (string), %d (integer), %f (float), etc. are called "con-
version specifiers". Here is an example:

      num1 = 4.123
      num2 = 5.678
      num3 = 123
      s    = 'hello'
      print ('num1 = %5.2f, num2 = %5.2f, num3 = %3d, and s =%6s.'%(num1, num2, num3, s))

      # OUTPUT: num1 =  4.12, num2 =  5.68, num3 = 123, and s = hello.

#                   LEFT-RIGHT JUSTIFY STRINGS and NUMBERS
#--Right justify a string in printing.
   s = "Hello"
   print ("(%10s)"%s)                       # output: (     Hello)
   print ( '(', s.rjust(10), ')', sep = '') # output: (     Hello)

#--Left justify a string in printing.
   s = "Hello"
   print ("(%-10s)"%s)                      # output: (Hello     )
   print ( '(', s.ljust(10), ')', sep = '') # output: (Hello     )

#--Center a string in printing.
   s = "Hello"
   print ( '(', s.center(10), ')', sep = '') # output: (  Hello  )

#--Changing the padding.
   s = "Hello"
   print ( '(', s.center(10, '^'), ')', sep = '') # output: (^^Hello^^^)

#--Right justify an integer in printing.
   n = 123; x = 12.34
   print ("(%10d)"%n)    # output: (       123)

#--Left justify an integer in printing.
   n = 123; x = 12.34
   print ("(%-10d)"%n)   # output: (123       )

#--Right justify a float in printing.
   print ("(%10.2f)"%x)  # output: (     12.34)

#--Left justify a float in printing.
   print ("(%-10.2f)"%x) # output: (12.34     )

#--Align values in a table (note the four columns).
   for n in range (48,51):
     print ("%3d\t%5.2f\t%3d\t%5.2f\t"%(n, sqrt(n), n + 50, sqrt(n + 50) ))

       output:           48	 6.93	 98	 9.90
                         49	 7.00	 99	 9.95
                         50	 7.07	100	10.00
"""
#######################################=#######################################
#                         PRINT FORMATTING FOR TABLES
#                     (ints, floats, chars and strings)
"""
#--Formatting integers (d = decimal) with %
   a = 7; b = 77
   print ("a = %3d"%a,  " b = %3d"%b)     # output: a =   7  b =  77
   print ("a = %03d"%a, " b = %03d"%b)    # output: a = 007  b = 077
   print ("a = %3d,       b = %3d"%(a,b)) # output: a =   7,       b =  77
   print ("a = %3d,       b = %3d"%(a,b)) # output: a =   7,       b =  77

#--Setting off by commas
   num = 1234567
   print('{0:,d}'.format(num)) # Output1,234,567. (Note the comma in '{0:,d}'.)
   exit()

#--Formatting numbers with decimal points (f = floats) with %.
   x = 7.7; y = 77.77; z = 7
   print ("x = %.4f"%x, " y = %2.4f"%y, " z = %2.4f"%z)
   # output from above: x = 7.7000  y = 77.7700  z = 7.0000

   print ("x = %0.4f"%x, " y = %02.4f"%y, " z = %02.4f"%z)
   # output from above: x = 7.7000  y = 77.7700  z = 7.0000

   print ("x = %.4f, y = %2.4f, z = %2.4f"%(x, y, z))
   # output from above: x = 7.7000  y = 77.7700  z = 7.0000

   print ("*%9.3f*"%-y)    # width includes decimal point and sign
   print ("*%09.3f*"%-y)   # left-zero padding
   print ("*%-9.3f*"%y)    # minus sign will left-align number
   print ("*%+9.3f*"%y)    # neg or pos sign will print

    output from the above four lines: *  -77.770*
                                         *-0077.770*
                                         *77.770   *
                                         *  +77.770*

#--Formatting hexidecimal (x = hexidecimal) numbers.
   a = 15; b = 16
   print ("a = %2X"%a, "a = %2x"%a, " b = %2X"%b) # a =  F a =  f  b = 10
   print (hex(15))          # Translates to hexidecimal:        15 = Oxf
   print (str(hex(15))[2:]) # Translates to hexidecimal string: 15 = f

#--Formatting octal (o = octal) numbers.
   x = 8; y = 9
   print ("x = %3o"%x)      # lower-case "o" only:               x = 10
   print (oct(15))          # Translates to hexidecimal:        15 = 0o17
   print (str(oct(15))[2:]) # Translates to hexidecimal string: 15 = 17

#--Formatting binary numbers. (No conversion specifier (%?) for binary exists.)
   n = 15
   print('bitlength = ', n.bit_length()) # bitlength =  4
   print (bin(15))          # Translates decimal to binary: 15 = Ob1111
   print (str(bin(15))[2:]) # Translates to binary string:  15 =   1111
   b = 111                  # considered in base 2
   print (int(str(111), 2)) # output: 7

#--Formatting characters (c = char) numbers of characters.
   ch = 65; kh = 'B'
   print ("ch = %3c"%ch, "kh = %3c"%kh) # output: ch =   A kh =   B

#--Formatting strings (s = str type).
   stng = 'Happy'
   print ("stng = %8s"%stng, "stng = %3s"%stng) # stng =    Happy stng = Happy

#--Print a list of strings.
   recent_presidents = ['George Bush', 'Bill Clinton', 'George W. Bush']
   print ('Presidents: %s.' % ', '.join(recent_presidents))
   # output: Presidents: George Bush, Bill Clinton, George W. Bush.
