"""+=================+=================*=================+=================+
   ||                                                                     ||
   ||                    INPUT, OUTPUT, and FORMATTING                    ||
   ||                                                                     ||
   +=================*===================================*=================+
      +=================-=========-=========-=========*=========-=========-=========-=================+
      ||                                                                                             ||
      ||                                                                                             ||
      +=================-=========-=========-=========*=========-=========-=========-=================+


PART I INPUT.
   For debugging purposes and most school problems, you do NOT want to do any kind on input--it is too in-
efficient and time consuming. Hard coding is the way to go. However, INPUT via dialogue box is done like this:

                str =       input ('Type in a string.')
                num = int  (input ('Type in an   int.'))
                flt = float(input ('Type in a  float.'))

   And a float cast will work for both integer and float types.

   NOTE WELL: Check input values for validity. Computer garu Steve McConnel notes that "Garbage in, garbage
out" is not a good policy. Better is "no garbage in." or "garbage in, try again", or "garbage in, error
message", or "garbage in, and the program aborts."

                         A dead program normally does a lot less damage than a
                         crippled one.--Andy Hunt and Dave Thomas, The Pragmatic
                         Programmer (Addison-Wesley, 1999), page ?.

   Below are two different ways to check-and-filter input. In the try/except/else block, either the EXCEPT
   part or the ELSE part will be run (but not both).

#---Input with a filter (method 1).
    flag = True
    s = 'Enter an integer:'
    while flag:
      try:
        num = int(input(s))
      except:
        s = 'That was NOT an integer! Try again.\nEnter an integer: '
      else:
        flag = False

#---Input with a filter (method 2).
    stng1 ="Enter code (= c), or decode (= d). ";
    stng2 = stng1
    while True:
       mode = input (stng2)
       if mode in ('c', 'd'): break
       stng3 = "BAD INPUT: " + mode + ". Try again. "
       stng2 = stng1 + stng3
    print ("OK, you are in.")
    --------------------------------------------------------------------------


PART II OUTPUT.
   Output in the console window is done like this:

                                print(1,2,3)

   You can control the optional string separator and the end-of-line char-
acter(s) like this:
                    print(1,2,3, sep = ', ', end = '\n')

   The '\n ' is the command for "move the cursor go to the beginning of the next
line".


PART III FORMATTING

#--Formatting integers (d = decimal) with %
   a = 7; b = 77
   print ("a = %3d"%a,  " b = %3d"%b)     # output: a =   7  b =  77
   print ("a = %03d"%a, " b = %03d"%b)    # output: a = 007  b = 077
   print ("a = %3d,       b = %3d"%(a,b)) # output: a =   7,       b =  77
   print ("a = %3d,       b = %3d"%(a,b)) # output: a =   7,       b =  77

#--Setting off by commas
   num = 1234567
   print('{0:,d}'.format(num)) # Output1,234,567. (Note the comma in '{0:,d}'.)
   exit()

#--Formatting numbers with decimal points (f = floats) with %.
   x = 7.7; y = 77.77; z = 7
   print ("x = %.4f"%x, " y = %2.4f"%y, " z = %2.4f"%z)
   # output from above: x = 7.7000  y = 77.7700  z = 7.0000

   print ("x = %0.4f"%x, " y = %02.4f"%y, " z = %02.4f"%z)
   # output from above: x = 7.7000  y = 77.7700  z = 7.0000


   print ("x = %.4f, y = %2.4f, z = %2.4f"%(x, y, z))
   # output from above: x = 7.7000  y = 77.7700  z = 7.0000

   print ("*%9.3f*"%-y)    # width includes decimal point and sign
   print ("*%09.3f*"%-y)   # left-zero padding
   print ("*%-9.3f*"%y)    # minus sign will left-align number
   print ("*%+9.3f*"%y)    # neg or pos sign will print

   output from the above four lines: *  -77.770*
                                     *-0077.770*
                                     *77.770   *
                                     *  +77.770*

#--Formatting hexidecimal (x = hexidecimal) numbers.
   a = 15; b = 16
   print ("a = %2X"%a, "a = %2x"%a, " b = %2X"%b) # a =  F a =  f  b = 10
   print (hex(15))          # Translates to hexidecimal:        15 = Oxf
   print (str(hex(15))[2:]) # Translates to hexidecimal string: 15 = f

#--Formatting octal (o = octal) numbers.
   x = 8; y = 9
   print ("x = %3o"%x)      # lower-case "o" only:               x = 10
   print (oct(15))          # Translates to hexidecimal:        15 = 0o17
   print (str(oct(15))[2:]) # Translates to hexidecimal string: 15 = 17

#--Formatting binary numbers. (No conversion specifier (%?) for binary exists.)
   n = 15
   print('bitlength = ', n.bit_length()) # bitlength =  4
   print (bin(15))          # Translates decimal to binary: 15 = Ob1111
   print (str(bin(15))[2:]) # Translates to binary string:  15 =   1111
   b = 111                  # considered in base 2
   print (int(str(111), 2)) # output: 7

#--Formatting characters (c = char) numbers of characters.
   ch = 65; kh = 'B'
   print ("ch = %3c"%ch, "kh = %3c"%kh) # output: ch =   A kh =   B

#--Formatting strings (s = str type).
   stng = 'Happy'
   print ("stng = %8s"%stng, "stng = %3s"%stng) # stng =    Happy stng = Happy

#--Print a list of strings.
   recent_presidents = ['George Bush', 'Bill Clinton', 'George W. Bush']
   print ('Presidents: %s.' % ', '.join(recent_presidents))
   # output: Presidents: George Bush, Bill Clinton, George W. Bush.
================================ END OF SECTION ================================
