"""   +=================-=========-=========-=========*=========-=========-=========-=================+
      ||                        QUEUES: LIFO (aka STACK), FIFO, and PRIORITY                         ||
      ||                              (SIMULATED AND REAL LINKED LISTS)                              ||
      +=================-=========-=========-=========*=========-=========-=========-=================+


   Python lists are not true linked lists. Consequently, adding and removing elements is inefficient. Never-
theless, Python lists work fine for most school assignments. If you insist on a true linked list (each node is
data with a pointer to the next data node), then you need the built-in queue class discussed in Part II. Below
you will see examples of both.


PART I. LISTS USED AS QUEUES.
   Lists are ideal for LIFO queues (AKA stacks), priority queues, but not so great for FIFO queues. The reason
is that any pop or append to the left side of a list forces all other elements to be moved over.

#---A LIFO queue (AKA STACK), with the top on the RIGHT (fast)
    stack = [1, 2, 3]
    stack.append(4)
    print (stack)    # output: [1, 2, 3, 4]
    x = stack.pop()
    print (stack)    # output: [1, 2, 3]
    print (x)        # output: 4

#---A PRIORITY queue to remove largest (pop()) or smallest (pop(0)) element. Again, we remove from the right-
#   side (fast), not the left-side (slow).
    queue = [1, 3, 2]
    queue.append(4)
    queue.sort()
    print (queue)    # output: [1, 2, 3, 4]
    x = queue.pop()  # use pop(0) to remove smallest element
    print (queue)    # output: [1, 2, 3]
    print (x)        # output: 4

#---A poor FIFO queue (enter on right (fast), exit on left (slow))
    queue = [1, 2, 3]
    queue.append(4)
    print (queue)    # output: [1, 2, 3, 4]
    x = queue.pop(0)
    print (queue)    # output: [2, 3, 4]
    print (x)        # output: 1

#---Another poor FIFO queue (enter on left(slow), exit on right (fast))
    queue = [1, 2, 3]
    queue.insert(0, 4)
    print (queue)    # output: [4, 1, 2, 3]
    x = queue.pop()
    print (queue)    # output: [4, 1, 2]
    print (x)        # output: 3

#---A fast FIFO queue. (best)
    from collections import deque
    queue = deque([1, 2, 3])
    queue.append(4)
    print (queue)   # output: deque([1, 2, 3, 4])
    queue.popleft() # there is no popright()
    print (queue)   # output: deque([2, 3, 4])

#---A CIRCULAR linked list is obtained by using %. Find the 15th element starting at 0.
    L = [0,1,2,3,4,5,6,7,8,9,]
    pos = 15 % len(L); print(L[pos]) # output: 5

#---A DOUBLY linked list (goes both directions). The element PRIOR to x[n] is x[n-1]. The element AFTER x[n]
    is x[n+1]. Perhaps a doubly linked list should also be circular.
================================================================================



PART II.
   Python does have a Built-in Queue class. The primary terms are put(), get(), empty(), qsize(), Queue(),
   LifoQue(), and PriorityQueue().

    from queue import * # Capitalized as Queue in Python versions 2.x.

#---FIFO Queue: push(bottom) and pop(top)
    q = Queue() # <-- This is a FIFO queue.
    Lst = ['A','B','C','D']
    for elt in Lst: q.put(elt)       # insert at back
    while not q.empty():
    print (q.get(), end = "")        # remove at front
    print()                          # output: ABCD

#---LIFO Queue: push(top) and pop(top) = STACK
    q = LifoQueue()                  # <-- This is a LIFO queue = a STACK.
    Lst = ['A','B','C','D']
    for elt in Lst: q.put(elt)       # insert at top
    print ( "Size = ", q.qsize() )   # output: Size =  4
    while not q.empty():
       print (q.get(), end = "")     # remove at top
    print()                          # output: DCBA

#---Priority Queue: append and sort list
    q = PriorityQueue()              # <-- This is a PRIORITY queue.
    Lst = ['C','D','A','B']
    for elt in Lst: q.put(elt)       # insert in order
    while not q.empty():
       print (q.get(), end = "")     # remove (small) at top
    print()                          # output: ABCD

#---Priority Queue, with priority attached:
    q = PriorityQueue()              # <-- This is a PRIORITY queue.
    Lst = [(1,'C'),(2,'D'),(4,'A'),(5,'B')]
    for elt in Lst: q.put(elt)       # insert in order
    while not q.empty():
       print (q.get()[1], end = "")  # remove (small) at top
    print()                          # output: CDAB
#=============================================================================================================
