"""   +=================-=========-=========-=========*=========-=========-=========-=================+
      ||                                          VARIABLES                                          ||
      ||                                                                                             ||
      +=================-=========-=========-=========*=========-=========-=========-=================+


   The biggest mistake you can make in selecting a variable, function name or file name is to choose a re-
served word, which corrupts the programming language. Try to choose descriptive identifiers--e.g.,

                    BAD:         doit(a,b)                <-- horrible!
                    BETTER:      reduce(a,b)              <-- verb.
                    BEST:        reduceFraction(a,b)      <-- verb-object.
                    EVEN BETTER: reduceFraction(num, den) <-- verb-object with descriptive parameters.

   The word IDENTIFIER means the name of a variable, constant, function, method, class, etc. A PARAMETER is a
variable passed to a function or method. The parameters in a header are FORMAL parameters (AKA dummy para-
meters). The parameters in a call are ACTUAL parameters. Avoid global variables, but accept global constants
(in all caps).

   It is possible to change the type of a variable by reassignment.

                                a = 1
                                a = 1.2
                                a = 'hello'
                                a = [1, 2, 3]

   Python is a DYNAMICALLY typed language. That means variable types are never explicitly declared by the
programmer. The computer determines the type of a variable by what value it is assigned. This dynamic typing
is a form of POLYMORPHISM. Of course, sometimes you need to know what data type you are working with. Python
allows the user to ask. Here are two methods:

                a = 2
                if type(a) in {int, float}:
                    print ('numeric') # output: numeric
                if type(a) == int:
                    print ('numeric') # output: numeric
                if isinstance (a, (int, float)):
                    print ('numeric') # output: numeric
                print (isinstance(bin(5), str)) # output: True

   Weak type-checking makes coding much easier. Python is great for short programs. A short program is called
a SCRIPT. Python is also good for reading other people's programs to learn about algorithms and coding tricks.
But if your programs start to go past 1000 lines, or if several people work on the same program, then this
lack of type checking by the compiler becomes a detriment.

How much memory does an object take? There is a way to find out.

      from sys import getsizeof
      print (getsizeof(1))        # output: 14 <--bits
      print (getsizeof(1.2))      # output: 16
      print (getsizeof('A'))      # output: 32
      print (getsizeof([1,]))     # output: 40 = size of container
      print (getsizeof([1,2,]))   # output: 44 = size of container
      print (getsizeof([1,2,3,])) # output: 48 = size of container
      print (sum ([getsizeof(x) for x in [1,2,3,]])) # output: 42 = contents of container

    Variable assignments can be listed on separate lines or on the same line with semi-colons separating them.
Swapping values is also easy. The following line does a swap: a, b = b, a.

                                        a = 1
                                        b = 2
                                        is the same as
                                        a = 1; b = 2
                                        is the same as
                                        a, b = 1, 2

                                        and

                                        a = 1
                                        b = 1
                                        is the same as
                                        a = b = 1

   A variable or constant (AKA an identifier) assigned outside of all functions is a global variable, and is
automatically referenced in all functions. However, global variables set us a GREAT TRAP. You can NOT MODIFY a
global variable in a function unless you list the global variable on the first line of the function with the
word "global".

      max = 100 # max is global
      min = 0   # min is global

      def doit():
          global max
          max += 1   # max is global
          min = 10   # min is local here

      def main():
          doit()
          print (min) # =   0, UNchanged
          print (max) # = 101,   changed.

    In the example above, the global identifier suddenly represents a LOCAL parameter of the same name--and no
warning or error message is generated (unless the variable is used before it is assigned a value). In general
AVOID GLOBAL VARIABLES--recursion can be an exception. However, globals are sometimes required in Tk graphics.
Global constants, however, are not discouraged. Advice: Write your constant identifiers in ALL CAPITALS.
